;; $Id: pv-mode.el,v 1.1 2001/05/27 19:55:54 jfine Exp $

;; pv-mode.el -- provide Instant Preview for (La)TeX files
;; Copyright 2001 by Jonathan Fine <jfine@activetex.org>

;; Instant Preview is free software distributed under the terms 
;; of the the GNU General Public License, 

;; This program is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License; either
;; version 2 of the License, or (at your option) any later version.

;; For more information about this program, send a message to
;; mailto:jfine@activetex.org

;; $Log: pv-mode.el,v $
;; Revision 1.1  2001/05/27 19:55:54  jfine
;; Initial revision
;;

;; The book "Writing GNU Emacs Extensions" by Bob Glickstein
;; (pub: O'Reilly, First Edition) has been used as a source.

;; Preview is an unusual mode, in that by design it adds no new
;; editing functionality to Emacs.  Instead, it adds display
;; functionality.

;; When Preview is enabled, one gets instant preview of the (La)TeX
;; file one is editing. Except only LaTeX implemented so far.

;; You will need to have "texd" installed, to use this package.
;; This version of pv.el is part of texd-0.1.  Please read the
;; installation instructions there.

;; set up the "preview" minor mode

(defvar pv-mode nil
  "Mode variable for preview minor mode.")

(defvar pv-xdvi-process nil
  "The xdvi process to be used, or nil.")

(make-variable-buffer-local 'pv-mode)

(defun pv-mode (&optional arg)
  "Preview minor mode."
  (interactive "P")
  (setq pv-mode
	(if (null arg)
	    (not pv-mode)
	  (> (prefix-numeric-value arg) 0)))
  (make-local-hook 'after-change-functions)
  (if pv-mode
      (progn
	(add-hook `after-change-functions 'pv-buffer nil t)
	(if (null (processp pv-xdvi-process))
	    (start-xdvi))
	)
    (remove-hook `after-change-functions 'pv-buffer t))
  )

(if (not (assq 'pv-mode minor-mode-alist))
    (setq minor-mode-alist
	  (cons `(pv-mode " Preview")
		minor-mode-alist)))

;; the preview function
(defun pv-buffer (&rest args)
  "Preview the current buffer"
  (if (null (processp pv-xdvi-process))
      (start-xdvi))
  (let ((write-region-annotate-functions '(pv--top-and-tail-file)))
    (write-region (point-min) (point-max)
		  (format "/tmp/%s/texd/dlatex.in" (getenv "LOGNAME"))
		  'nil			; don't append
		  'no-message		; don't print "Wrote file" message
		  )
    )
  )

;; top and tail strings (buffer specific?)
(defun pv--top-and-tail-file (start end)
  "Wrapping text placed around buffer contents"
  (list `(,start . ,(format "\n\\BeginChop{%d}{%d}\n" 
			    (emacs-pid) 
			    (process-id pv-xdvi-process)
			    ))
	`(,end . "\n\n\\EndChop\n")
	)
  )

;; useful to place this under the control of emacs
(defun start-xdvi()
  "Start xdvi (and restart it if it exist abnormally)"
  (interactive)
  (let ((xdvi-file (format "/tmp/jfine/dvi/%d.dvi" (emacs-pid))))
    (copy-file "~/.texd/null.dvi"  xdvi-file 't)
    (message "Starting xdvi ...")
    (setq pv-xdvi-process (start-process 
			   "xdvi" 
			   nil 
			   "xdvi"
			   "-expert" "-geometry" "512x768+512+0"
			   "-paper" "1ptx1pt"
			   "-bg" "DarkSlateGray"
			   "-fg" "Wheat"
			   "-hl" "DarkSlateGray"
			   "-s" "2"
			   xdvi-file)))
  (set-process-sentinel pv-xdvi-process `pv-xdvi-sentinel)
  't)

;; xdvi can die if it access a changing dvi file
(defun pv-xdvi-sentinel (process string)
  "Restart xdvi if it exits abnormally"
  (if (zerop (process-exit-status process))
      (setq pv-xdvi-process nil)
    (start-xdvi)
    )
  )

;; pv.el ends here
